==> tidyfiles-0.11.0.tar.gz: tidyfiles-0.11.0/tests/test_security.py <==
import pytest
import sys
import os
from pathlib import Path
from tidyfiles.security import SystemSecurity


@pytest.fixture
def system_path():
    """Fixture for system path based on platform."""
    return Path("C:/Windows" if sys.platform == "win32" else "/etc")


@pytest.fixture
def test_file(tmp_path):
    """Fixture for temporary test file with cleanup."""
    test_file = tmp_path / "test.txt"
    test_file.touch()
    yield test_file
    if test_file.exists():
        os.chmod(test_file, 0o644)  # Ensure writeable for cleanup
        test_file.unlink()


@pytest.fixture
def platform_system_paths():
    """Fixture providing expected system paths for current platform."""
    if sys.platform == "win32":
        return {Path("C:/Windows"), Path("C:/Program Files")}
    elif sys.platform == "darwin":
        return {Path("/System"), Path("/Library")}
    else:  # Linux/Unix
        return {Path("/etc"), Path("/usr")}


class TestSystemPathDetection:
    """Tests for system path detection functionality."""

    def test_system_path_detection(self, system_path):
        """Test that system paths are correctly identified as unsafe."""
        is_safe, reason = SystemSecurity.is_safe_path(system_path)
        assert not is_safe
        assert "system directory" in reason or "not writable" in reason

    def test_is_system_path_with_relative_path(self):
        """Test that relative paths are not incorrectly flagged as system paths."""
        relative_path = Path("usr/local/bin")
        is_system = SystemSecurity.is_system_path(relative_path)
        assert not is_system


def test_safe_path_validation(tmp_path):
    """Test path safety validation for safe paths."""
    is_safe, reason = SystemSecurity.is_safe_path(tmp_path)
    assert is_safe
    assert reason == ""


def test_validate_path_raises_error(system_path):
    """Test that validate_path raises error for system paths."""
    with pytest.raises(ValueError) as exc_info:
        SystemSecurity.validate_path(system_path)
    assert "system directory" in str(exc_info.value) or "not writable" in str(
        exc_info.value
    )


class TestPathPermissions:
    """Tests for path permission handling."""

    def test_non_writable_path(self, test_file):
        """Test detection of non-writable paths."""
        os.chmod(test_file, 0o444)
        is_safe, reason = SystemSecurity.is_safe_path(test_file)
        assert not is_safe
        assert "not writable" in reason


def test_parent_directory_permissions(tmp_path):
    """Test parent directory permission checking."""
    test_dir = tmp_path / "test_dir"
    test_dir.mkdir()
    test_file = test_dir / "test.txt"

    try:
        # Make parent directory read-only
        os.chmod(test_dir, 0o444)

        is_safe, reason = SystemSecurity.is_safe_path(test_file)
        assert not is_safe
        assert "Parent directory" in reason
        assert "not writable" in reason
    finally:
        # Restore permissions to allow cleanup
        os.chmod(test_dir, 0o755)


def test_validate_path_without_raising(system_path):
    """Test validate_path with raise_error=False."""
    # Should not raise an exception
    SystemSecurity.validate_path(system_path, raise_error=False)


def test_system_paths_all_platforms():
    """Test system paths detection across all platforms."""
    paths = SystemSecurity.get_system_paths()
    assert isinstance(paths, set)
    assert len(paths) > 0
    assert all(isinstance(p, Path) for p in paths)


@pytest.mark.parametrize(
    "path_str",
    [
        "/nonexistent/path",
        "C:/nonexistent/path" if sys.platform == "win32" else "/nonexistent/path",
    ],
)
def test_is_safe_path_nonexistent(path_str):
    """Test is_safe_path with nonexistent paths."""
    path = Path(path_str)
    is_safe, reason = SystemSecurity.is_safe_path(path)
    assert not is_safe
    assert "Parent directory" in reason


def test_is_safe_path_with_macos_hidden_file(tmp_path):
    """Test is_safe_path with macOS hidden system files."""
    hidden_file = tmp_path / "._hidden_file"
    hidden_file.touch()

    is_safe, reason = SystemSecurity.is_safe_path(hidden_file)
    if sys.platform == "darwin":
        assert not is_safe
        assert "macOS system file" in reason
    else:
        assert is_safe
        assert reason == ""


def test_is_safe_path_with_windows_special_file(tmp_path):
    """Test is_safe_path with Windows special files."""
    special_file = tmp_path / "~tempfile"
    special_file.touch()

    is_safe, reason = SystemSecurity.is_safe_path(special_file)
    if sys.platform == "win32":
        assert not is_safe
        assert "Windows special file" in reason
    else:
        assert is_safe
        assert reason == ""


def test_is_safe_path_with_exception(monkeypatch):
    """Test is_safe_path handling of unexpected exceptions."""

    def mock_is_system_path(*args, **kwargs):
        raise PermissionError("Mock permission error")

    monkeypatch.setattr(SystemSecurity, "is_system_path", mock_is_system_path)
    path = Path("/some/path")

    is_safe, reason = SystemSecurity.is_safe_path(path)
    assert not is_safe
    assert "Unable to verify path safety" in reason or "Permission denied" in reason


def test_is_safe_path_with_access_error(monkeypatch):
    """Test is_safe_path handling of access permission errors."""

    def mock_access(*args, **kwargs):
        raise PermissionError("Mock access error")

    original_exists = Path.exists

    def mock_exists(self, *args, **kwargs):
        if str(self) in ["/some/path", "/some"]:  # Mock both path and parent
            return True
        return original_exists(self)

    monkeypatch.setattr(os, "access", mock_access)
    monkeypatch.setattr(Path, "exists", mock_exists)
    path = Path("/some/path")

    is_safe, reason = SystemSecurity.is_safe_path(path)
    assert not is_safe
    # The function checks parent exists first, which might short-circuit before our mocked access error
    assert (
        "Unable to verify path safety" in reason
        or "not exist" in reason
        or "not writable" in reason
    )


def test_is_system_path_with_exception(monkeypatch):
    """Test is_system_path handling of resolution errors."""

    def mock_resolve(*args, **kwargs):
        raise OSError("Mock resolution error")

    monkeypatch.setattr(Path, "resolve", mock_resolve)
    path = Path("/some/path")

    assert SystemSecurity.is_system_path(path) is True


@pytest.mark.parametrize(
    "platform,expected_paths",
    [
        ("win32", {Path("C:/Windows"), Path("C:/Program Files")}),
        ("darwin", {Path("/System"), Path("/Library")}),
        ("linux", {Path("/etc"), Path("/usr")}),
    ],
)
def test_get_system_paths_per_platform(monkeypatch, platform, expected_paths):
    """Test system paths retrieval for different platforms."""
    monkeypatch.setattr(sys, "platform", platform)
    paths = SystemSecurity.get_system_paths()
    assert expected_paths.issubset(paths)


def test_windows_system_paths_with_custom_drive(monkeypatch):
    """Test Windows system paths with non-default system drive."""
    monkeypatch.setattr(sys, "platform", "win32")
    monkeypatch.setenv("SystemDrive", "D:")

    paths = SystemSecurity.get_windows_system_paths()
    assert Path("D:/Windows") in paths
    assert Path("D:/Program Files") in paths


def test_is_system_path_with_symlink(tmp_path):
    """Test is_system_path with symbolic links pointing to system paths."""
    if sys.platform == "win32":
        system_path = Path("C:/Windows")
    else:
        system_path = Path("/etc")

    link_path = tmp_path / "system_link"
    try:
        link_path.symlink_to(system_path)
        assert SystemSecurity.is_system_path(link_path)
    except OSError:
        pytest.skip("Symbolic link creation requires elevated privileges")


class TestEdgeCases:
    """Tests for edge cases and boundary conditions."""

    def test_is_safe_path_with_empty_path(self):
        """Test handling of empty path."""
        # Empty path is handled by CLI validation, so security check assumes it's safe
        result = SystemSecurity.is_safe_path(Path())
        assert result[0]  # Should be safe since CLI prevents empty paths

    def test_is_safe_path_with_root(self):
        """Test handling of root directory."""
        is_safe, reason = SystemSecurity.is_safe_path(Path("/"))
        assert not is_safe
        # The actual error could be either about system directory or not being writable
        assert "not writable" in reason or "system directory" in reason

    def test_path_with_invalid_chars(self, tmp_path):
        """Test paths with invalid characters."""
        # Note: On some systems, null character might be handled without error
        # We'll skip the null character test on platforms where it doesn't raise
        if sys.platform == "win32":
            invalid_chars = '<>:"|?*'
        else:
            # Skip the test for platforms where null bytes don't cause errors in Path creation
            pytest.skip("Skipping invalid character test on non-Windows platforms")
            return

        for char in invalid_chars:
            try:
                path = tmp_path / f"test{char}file.txt"
                with pytest.raises((OSError, ValueError)):
                    # This should raise an error
                    SystemSecurity.is_safe_path(path)
            except (OSError, ValueError):
                # Expected behavior - test passes
                pass


class TestSecurityCases:
    """Tests for security-critical scenarios."""

    def test_path_traversal_attempts(self, tmp_path):
        """Test protection against path traversal attacks."""
        # Create a resolved path to simulate path traversal
        traversal_paths = [
            tmp_path / "../../../etc/passwd",
            tmp_path / "..\\..\\Windows\\System32",
            tmp_path / "folder/../../../etc/shadow",
        ]
        for path in traversal_paths:
            # Note: Path normalization in Python means these paths will resolve to their actual targets
            # So we need to check the functionality differently

            # First, check if Path.resolve() actually leads to a system path
            try:
                resolved_path = path.resolve()
                if SystemSecurity.is_system_path(resolved_path):
                    # This is the actual security check we care about
                    is_safe, reason = SystemSecurity.is_safe_path(path)
                    assert not is_safe
                    assert (
                        "system directory" in reason
                        or "not writable" in reason
                        or "does not exist" in reason
                    )
            except (OSError, RuntimeError):
                # If resolving the path itself fails, the test is moot
                pass

    def test_symlink_loop_detection(self, tmp_path):
        """Test handling of symbolic link loops that could cause infinite recursion."""
        link1 = tmp_path / "link1"
        link2 = tmp_path / "link2"
        try:
            # Create a normal file first
            link1.touch()
            link2.symlink_to(link1)

            # Test a simple symlink first
            is_safe, reason = SystemSecurity.is_safe_path(link2)

            # Now try to create a symlink loop
            os.unlink(link1)
            link1.symlink_to(link2)

            # The symlink loop should be detected during path resolution
            # and handled appropriately by the implementation
            try:
                SystemSecurity.is_safe_path(link1)
                # If the method handles symlink loops, the result should indicate unsafe
                # but we won't strictly assert this as the implementation might vary
            except (OSError, RuntimeError):
                # If it raises an exception, that's also acceptable
                pass

        except OSError:
            pytest.skip("Symbolic link creation requires elevated privileges")

    def test_device_files(self):
        """Test handling of device files which should be protected."""
        device_paths = ["/dev/null", "/dev/random", "/dev/sda"]
        for dev_path in device_paths:
            path = Path(dev_path)
            if path.exists():
                is_safe, reason = SystemSecurity.is_safe_path(path)
                assert not is_safe
                # The reason could be about being a system directory or not writable
                assert "not writable" in reason or "system directory" in reason


class TestErrorHandling:
    """Tests for error handling scenarios."""

    @pytest.mark.parametrize(
        "invalid_input",
        [
            123,  # Only numeric types should raise TypeError
            object(),  # Custom objects should raise TypeError
        ],
    )
    def test_invalid_input_types(self, invalid_input):
        """Test handling of invalid input types."""
        with pytest.raises((TypeError, ValueError)):
            # Path constructor itself will raise TypeError for these inputs
            SystemSecurity.is_safe_path(Path(invalid_input))

    # Separate test for empty strings which are valid Path inputs
    def test_empty_string_input(self):
        """Test handling of empty string input."""
        # Empty string path is handled by CLI validation, so security check assumes it's safe
        is_safe, reason = SystemSecurity.is_safe_path(Path(""))
        assert is_safe  # Should be safe since CLI prevents empty paths

    # Separate test for string paths
    def test_string_path_input(self):
        """Test handling of string path input that may not exist."""
        # Non-existent paths are handled by CLI validation, so security check assumes it's safe
        path = Path("not_a_path")
        is_safe, reason = SystemSecurity.is_safe_path(path)
        assert is_safe  # Should be safe since CLI prevents non-existent paths

    def test_permission_denied_handling(self, monkeypatch):
        """Test handling of permission denied errors during path safety checks."""

        def mock_access(*args, **kwargs):
            raise PermissionError("Permission denied")

        def mock_exists(self, *args, **kwargs):
            return True

        monkeypatch.setattr(os, "access", mock_access)
        monkeypatch.setattr(Path, "exists", mock_exists)
        path = Path("/some/path")

        is_safe, reason = SystemSecurity.is_safe_path(path)
        assert not is_safe
        assert "Unable to verify path safety" in reason or "Permission denied" in reason

==> tidyfiles-0.12.0.tar.gz: tidyfiles-0.12.0/tests/test_security.py <==
import pytest
import sys
import os
from pathlib import Path
from tidyfiles.security import SystemSecurity


@pytest.fixture
def system_path():
    """Fixture for system path based on platform."""
    return Path("C:/Windows" if sys.platform == "win32" else "/etc")


@pytest.fixture
def test_file(tmp_path):
    """Fixture for temporary test file with cleanup."""
    test_file = tmp_path / "test.txt"
    test_file.touch()
    yield test_file
    if test_file.exists():
        os.chmod(test_file, 0o644)  # Ensure writeable for cleanup
        test_file.unlink()


@pytest.fixture
def platform_system_paths():
    """Fixture providing expected system paths for current platform."""
    if sys.platform == "win32":
        return {Path("C:/Windows"), Path("C:/Program Files")}
    elif sys.platform == "darwin":
        return {Path("/System"), Path("/Library")}
    else:  # Linux/Unix
        return {Path("/etc"), Path("/usr")}


class TestSystemPathDetection:
    """Tests for system path detection functionality."""

    def test_system_path_detection(self, system_path):
        """Test that system paths are correctly identified as unsafe."""
        is_safe, reason = SystemSecurity.is_safe_path(system_path)
        assert not is_safe
        assert "system directory" in reason or "not writable" in reason

    def test_is_system_path_with_relative_path(self):
        """Test that relative paths are not incorrectly flagged as system paths."""
        relative_path = Path("usr/local/bin")
        is_system = SystemSecurity.is_system_path(relative_path)
        assert not is_system


def test_safe_path_validation(tmp_path):
    """Test path safety validation for safe paths."""
    is_safe, reason = SystemSecurity.is_safe_path(tmp_path)
    assert is_safe
    assert reason == ""


def test_validate_path_raises_error(system_path):
    """Test that validate_path raises error for system paths."""
    with pytest.raises(ValueError) as exc_info:
        SystemSecurity.validate_path(system_path)
    assert "system directory" in str(exc_info.value) or "not writable" in str(
        exc_info.value
    )


class TestPathPermissions:
    """Tests for path permission handling."""

    def test_non_writable_path(self, test_file):
        """Test detection of non-writable paths."""
        os.chmod(test_file, 0o444)
        is_safe, reason = SystemSecurity.is_safe_path(test_file)
        assert not is_safe
        assert "not writable" in reason


def test_parent_directory_permissions(tmp_path):
    """Test parent directory permission checking."""
    test_dir = tmp_path / "test_dir"
    test_dir.mkdir()
    test_file = test_dir / "test.txt"

    try:
        # Make parent directory read-only
        os.chmod(test_dir, 0o444)

        is_safe, reason = SystemSecurity.is_safe_path(test_file)
        assert not is_safe
        assert "Parent directory" in reason
        assert "not writable" in reason
    finally:
        # Restore permissions to allow cleanup
        os.chmod(test_dir, 0o755)


def test_validate_path_without_raising(system_path):
    """Test validate_path with strict=False."""
    # Should not raise an exception when strict is False
    SystemSecurity.validate_path(system_path, strict=False)


def test_system_paths_all_platforms():
    """Test system paths detection across all platforms."""
    paths = SystemSecurity.get_system_paths()
    assert isinstance(paths, set)
    assert len(paths) > 0
    assert all(isinstance(p, Path) for p in paths)


@pytest.mark.parametrize(
    "path_str",
    [
        "/nonexistent/path",
        "C:/nonexistent/path" if sys.platform == "win32" else "/nonexistent/path",
    ],
)
def test_is_safe_path_nonexistent(path_str):
    """Test is_safe_path with nonexistent paths."""
    path = Path(path_str)
    is_safe, reason = SystemSecurity.is_safe_path(path)
    assert not is_safe
    assert "Parent directory" in reason


def test_is_safe_path_with_macos_hidden_file(tmp_path):
    """Test is_safe_path with macOS hidden system files."""
    hidden_file = tmp_path / "._hidden_file"
    hidden_file.touch()

    is_safe, reason = SystemSecurity.is_safe_path(hidden_file)
    if sys.platform == "darwin":
        assert not is_safe
        assert "macOS system file" in reason
    else:
        assert is_safe
        assert reason == ""


def test_is_safe_path_with_windows_special_file(tmp_path):
    """Test is_safe_path with Windows special files."""
    special_file = tmp_path / "~tempfile"
    special_file.touch()

    is_safe, reason = SystemSecurity.is_safe_path(special_file)
    if sys.platform == "win32":
        assert not is_safe
        assert "Windows special file" in reason
    else:
        assert is_safe
        assert reason == ""


def test_is_safe_path_with_exception(monkeypatch):
    """Test is_safe_path handling of unexpected exceptions."""

    def mock_is_system_path(*args, **kwargs):
        raise PermissionError("Mock permission error")

    monkeypatch.setattr(SystemSecurity, "is_system_path", mock_is_system_path)
    path = Path("/some/path")

    is_safe, reason = SystemSecurity.is_safe_path(path)
    assert not is_safe
    assert "Unable to verify path safety" in reason or "Permission denied" in reason


def test_is_safe_path_with_access_error(monkeypatch):
    """Test is_safe_path handling of access permission errors."""

    def mock_access(*args, **kwargs):
        raise PermissionError("Mock access error")

    original_exists = Path.exists

    def mock_exists(self, *args, **kwargs):
        if str(self) in ["/some/path", "/some"]:  # Mock both path and parent
            return True
        return original_exists(self)

    monkeypatch.setattr(os, "access", mock_access)
    monkeypatch.setattr(Path, "exists", mock_exists)
    path = Path("/some/path")

    is_safe, reason = SystemSecurity.is_safe_path(path)
    assert not is_safe
    # The function checks parent exists first, which might short-circuit before our mocked access error
    assert (
        "Unable to verify path safety" in reason
        or "not exist" in reason
        or "not writable" in reason
    )


def test_is_system_path_with_exception(monkeypatch):
    """Test is_system_path handling of resolution errors."""

    def mock_resolve(*args, **kwargs):
        raise OSError("Mock resolution error")

    monkeypatch.setattr(Path, "resolve", mock_resolve)
    path = Path("/some/path")

    assert SystemSecurity.is_system_path(path) is True


@pytest.mark.parametrize(
    "platform,expected_paths",
    [
        ("win32", {Path("C:/Windows"), Path("C:/Program Files")}),
        ("darwin", {Path("/System"), Path("/Library")}),
        ("linux", {Path("/etc"), Path("/usr")}),
    ],
)
def test_get_system_paths_per_platform(monkeypatch, platform, expected_paths):
    """Test system paths retrieval for different platforms."""
    monkeypatch.setattr(sys, "platform", platform)
    paths = SystemSecurity.get_system_paths()
    assert expected_paths.issubset(paths)


def test_windows_system_paths_with_custom_drive(monkeypatch):
    """Test Windows system paths with non-default system drive."""
    monkeypatch.setattr(sys, "platform", "win32")
    monkeypatch.setenv("SystemDrive", "D:")

    paths = SystemSecurity.get_windows_system_paths()
    assert Path("D:/Windows") in paths
    assert Path("D:/Program Files") in paths


def test_is_system_path_with_symlink(tmp_path):
    """Test is_system_path with symbolic links pointing to system paths."""
    if sys.platform == "win32":
        system_path = Path("C:/Windows")
    else:
        system_path = Path("/etc")

    link_path = tmp_path / "system_link"
    try:
        link_path.symlink_to(system_path)
        assert SystemSecurity.is_system_path(link_path)
    except OSError:
        pytest.skip("Symbolic link creation requires elevated privileges")


class TestEdgeCases:
    """Tests for edge cases and boundary conditions."""

    def test_is_safe_path_with_empty_path(self):
        """Test handling of empty path."""
        # Empty path is handled by CLI validation, so security check assumes it's safe
        result = SystemSecurity.is_safe_path(Path())
        assert result[0]  # Should be safe since CLI prevents empty paths

    def test_is_safe_path_with_root(self):
        """Test handling of root directory."""
        is_safe, reason = SystemSecurity.is_safe_path(Path("/"))
        assert not is_safe
        # The actual error could be either about system directory or not being writable
        assert "not writable" in reason or "system directory" in reason

    def test_path_with_invalid_chars(self, tmp_path):
        """Test paths with invalid characters."""
        # Note: On some systems, null character might be handled without error
        # We'll skip the null character test on platforms where it doesn't raise
        if sys.platform == "win32":
            invalid_chars = '<>:"|?*'
        else:
            # Skip the test for platforms where null bytes don't cause errors in Path creation
            pytest.skip("Skipping invalid character test on non-Windows platforms")
            return

        for char in invalid_chars:
            try:
                path = tmp_path / f"test{char}file.txt"
                with pytest.raises((OSError, ValueError)):
                    # This should raise an error
                    SystemSecurity.is_safe_path(path)
            except (OSError, ValueError):
                # Expected behavior - test passes
                pass


class TestSecurityCases:
    """Tests for security-critical scenarios."""

    def test_path_traversal_attempts(self, tmp_path):
        """Test protection against path traversal attacks."""
        # Create a resolved path to simulate path traversal
        traversal_paths = [
            tmp_path / "../../../etc/passwd",
            tmp_path / "..\\..\\Windows\\System32",
            tmp_path / "folder/../../../etc/shadow",
        ]
        for path in traversal_paths:
            # Note: Path normalization in Python means these paths will resolve to their actual targets
            # So we need to check the functionality differently

            # First, check if Path.resolve() actually leads to a system path
            try:
                resolved_path = path.resolve()
                if SystemSecurity.is_system_path(resolved_path):
                    # This is the actual security check we care about
                    is_safe, reason = SystemSecurity.is_safe_path(path)
                    assert not is_safe
                    assert (
                        "system directory" in reason
                        or "not writable" in reason
                        or "does not exist" in reason
                    )
            except (OSError, RuntimeError):
                # If resolving the path itself fails, the test is moot
                pass

    def test_symlink_loop_detection(self, tmp_path):
        """Test handling of symbolic link loops that could cause infinite recursion."""
        link1 = tmp_path / "link1"
        link2 = tmp_path / "link2"
        try:
            # Create a normal file first
            link1.touch()
            link2.symlink_to(link1)

            # Test a simple symlink first
            is_safe, reason = SystemSecurity.is_safe_path(link2)

            # Now try to create a symlink loop
            os.unlink(link1)
            link1.symlink_to(link2)

            # The symlink loop should be detected during path resolution
            # and handled appropriately by the implementation
            try:
                SystemSecurity.is_safe_path(link1)
                # If the method handles symlink loops, the result should indicate unsafe
                # but we won't strictly assert this as the implementation might vary
            except (OSError, RuntimeError):
                # If it raises an exception, that's also acceptable
                pass

        except OSError:
            pytest.skip("Symbolic link creation requires elevated privileges")

    def test_device_files(self):
        """Test handling of device files which should be protected."""
        device_paths = ["/dev/null", "/dev/random", "/dev/sda"]
        for dev_path in device_paths:
            path = Path(dev_path)
            if path.exists():
                is_safe, reason = SystemSecurity.is_safe_path(path)
                assert not is_safe
                # The reason could be about being a system directory or not writable
                assert "not writable" in reason or "system directory" in reason


class TestErrorHandling:
    """Tests for error handling scenarios."""

    @pytest.mark.parametrize(
        "invalid_input",
        [
            123,  # Only numeric types should raise TypeError
            object(),  # Custom objects should raise TypeError
        ],
    )
    def test_invalid_input_types(self, invalid_input):
        """Test handling of invalid input types."""
        with pytest.raises((TypeError, ValueError)):
            # Path constructor itself will raise TypeError for these inputs
            SystemSecurity.is_safe_path(Path(invalid_input))

    # Separate test for empty strings which are valid Path inputs
    def test_empty_string_input(self):
        """Test handling of empty string input."""
        # Empty string path is handled by CLI validation, so security check assumes it's safe
        is_safe, reason = SystemSecurity.is_safe_path(Path(""))
        assert is_safe  # Should be safe since CLI prevents empty paths

    # Separate test for string paths
    def test_string_path_input(self):
        """Test handling of string path input that may not exist."""
        # Non-existent paths are handled by CLI validation, so security check assumes it's safe
        path = Path("not_a_path")
        is_safe, reason = SystemSecurity.is_safe_path(path)
        assert is_safe  # Should be safe since CLI prevents non-existent paths

    def test_permission_denied_handling(self, monkeypatch):
        """Test handling of permission denied errors during path safety checks."""

        def mock_access(*args, **kwargs):
            raise PermissionError("Permission denied")

        def mock_exists(self, *args, **kwargs):
            return True

        monkeypatch.setattr(os, "access", mock_access)
        monkeypatch.setattr(Path, "exists", mock_exists)
        path = Path("/some/path")

        is_safe, reason = SystemSecurity.is_safe_path(path)
        assert not is_safe
        assert "Unable to verify path safety" in reason or "Permission denied" in reason
